//
// cons.c
//
// Copyright (c) 2001 Michael Ringgaard. All rights reserved.
//
// Console device driver
//

#include <os/krnl.h>

devno_t consdev = NODEV;
struct event enable_output;
static int cursoff = 0;

static int console_ioctl(struct dev *dev, int cmd, void *args)
{
  switch (cmd)
  {
    case IOCTL_GETDEVSIZE:
      return 0;

    case IOCTL_GETBLKSIZE:
      return 1;
  }
  
  return -1;
}

static int console_read(struct dev *dev, void *buffer, size_t count, blkno_t blkno)
{
  char *p = (char *) buffer;
  int ch;
  int left = count;

  while (left > 0)
  {
    if (cursoff)
    {
      cursoff = 0;
      show_cursor();
    }

    ch = getch();
    
    if (ch == 19)
      reset_event(&enable_output);
    else
    {
      set_event(&enable_output);

      *p++ = ch;
      left--;
    }
  }

  return count; 
}

static int console_write(struct dev *dev, void *buffer, size_t count, blkno_t blkno)
{
  wait_for_object(&enable_output);

  if (!cursoff)
  {
    cursoff = 1;
    hide_cursor();
  }

  print_buffer((char *) buffer, count);

  return count;
}

struct driver console_driver =
{
  "Console", 
  DEV_TYPE_STREAM,
  console_ioctl,
  console_read,
  console_write
};

void init_cons()
{
  init_event(&enable_output, 1, 1);
  dev_make("console", &console_driver, NULL);
  consdev = dev_open("console");
}

void kprintf(const char *fmt,...)
{
  va_list args;
  char buffer[1024];

  va_start(args, fmt);
  vsprintf(buffer, fmt, args);
  va_end(args);
  
  if (consdev == NODEV)
    print_string(buffer);
  else
    dev_write(consdev, buffer, strlen(buffer), 0);
}
