//
// object.h
//
// Copyright (c) 2001 Michael Ringgaard. All rights reserved.
//
// Object manager
//

#ifndef THREAD_H
#define THREAD_H

#define WAIT_ALL 0
#define WAIT_ANY 1

#define OBJECT_ANY       -1

#define OBJECT_THREAD    0
#define OBJECT_EVENT     1
#define OBJECT_TIMER     2
#define OBJECT_MUTEX     3
#define OBJECT_SEMAPHORE 4
#define OBJECT_FILE      5

#define THREAD_STATE_INITIALIZED 0
#define THREAD_STATE_READY       1
#define THREAD_STATE_STANDBY     2
#define THREAD_STATE_RUNNING     3
#define THREAD_STATE_WAITING     4
#define THREAD_STATE_TRANSITION  5
#define THREAD_STATE_TERMINATED  6

struct thread;
struct waitblock;

typedef void *object_t;

struct object
{
  unsigned short type;
  short signaled;

  int handle_count;

  struct waitblock *waitlist_head;
  struct waitblock *waitlist_tail;
};

struct waitblock
{
  struct thread *thread;
  struct object *object;

  unsigned short waittype;
  unsigned short waitkey;

  struct waitblock *next;

  struct waitblock *next_wait;
  struct waitblock *prev_wait;
};

struct event
{
  struct object object;
  int manual_reset;
};

struct sem
{
  struct object object;
  unsigned int count;
};

struct mutex
{
  struct object object;
  struct thread *owner;
  int recursion;
};

struct timer
{
  struct object object;
  unsigned int expires;

  struct timer *next_timer;
  struct timer *prev_timer;
};

struct thread
{
  struct object object;

  int state;
  int priority;
  int id;
  handle_t self;
  struct tib *tib;

  struct thread *next_ready;
  struct waitblock *waitlist;
  unsigned char waittype;

  struct thread *next_buffer_waiter;

  struct timer timeout;
  struct waitblock auxwb0;
};

void init_object(struct object *o, int type);
int close_object(struct object *o);

void init_thread(struct thread *t, int priority);

void init_event(struct event *e, int manual_reset, int initial_state);
void pulse_event(struct event *e);
void set_event(struct event *e);
void reset_event(struct event *e);

void init_sem(struct sem *s, unsigned int initial_count);
unsigned int release_sem(struct sem *s, unsigned int count);

void init_mutex(struct mutex *m, int owned);
void release_mutex(struct mutex *m);

void handle_timer_expiry(unsigned int ticks);
void init_timer(struct timer *t, unsigned int expires);
void modify_timer(struct timer *t, unsigned int expires);
void cancel_timer(struct timer *t);

void wait_for_object(object_t hobj);
void sleep(unsigned int ticks);

handle_t halloc(struct object *o);
int hfree(handle_t h);
int hrel(handle_t h);
struct object *hlock(handle_t h, int type);

#endif