//
// sched.h
//
// Copyright (c) 2001 Michael Ringgaard. All rights reserved.
// 
// Task scheduler
//

#ifndef SCHED_H
#define SCHED_H

typedef void (*taskproc_t)(void *arg);
typedef void (*dpcproc_t)(void *arg);

#define THREAD_PRIORITY_LEVELS   16

#define PRIORITY_IDLE   0
#define PRIORITY_NORMAL 8

#define PAGES_PER_TCB     2

#define TCBSIZE           (PAGES_PER_TCB * PAGESIZE)
#define TCBMASK           (~(TCBSIZE - 1))
#define TCBESP            (TCBSIZE - 4)

struct tcb
{
  struct thread thread;
  unsigned long stack[(TCBSIZE - sizeof(struct thread)) / sizeof(unsigned long) - 1];
  unsigned long *esp;
};

struct dpc
{
  dpcproc_t proc;
  void *arg;
  struct dpc *next;
};

extern int resched;
extern int idle;
extern struct thread *idlethread;

void mark_thread_ready(struct thread *t);
void mark_thread_running();

struct thread *current_thread();
struct thread *create_task(taskproc_t task, void *arg, int priority);
int init_user_thread(struct thread *t, unsigned long stack_reserved, unsigned long stack_committed);

void init_dpc(struct dpc *dpc);
void queue_dpc(struct dpc *dpc, dpcproc_t proc, void *arg);
void queue_irq_dpc(struct dpc *dpc, dpcproc_t proc, void *arg);

void idle_task();
void yield();
void dispatch_dpc_queue();
void dispatch();

void init_sched();

#endif
