//
// dev.c
//
// Copyright (c) 2001 Michael Ringgaard. All rights reserved.
//
// Device Manager
//

#include <os/krnl.h>

#define MAX_DEVICES             32

struct dev *devtab[MAX_DEVICES];
unsigned int num_devices = 0;

struct dev *device(devno_t devno)
{
  if (devno < 0 || devno >= num_devices) return NULL;
  return devtab[devno];
}

devno_t dev_make(char *name, struct driver *driver, void *privdata)
{
  struct dev *dev;
  devno_t devno;

  if (num_devices == MAX_DEVICES) panic("too many devices");
  devno = num_devices++;

  dev = (struct dev *) kmalloc(sizeof(struct dev));
  devtab[devno] = dev;

  dev->name = name;
  dev->driver = driver;
  dev->privdata = privdata;
  dev->refcnt = 0;

  return devno;
}

devno_t dev_open(char *name)
{
  devno_t devno;

  for (devno = 0; devno < num_devices; devno++)
  {
    if (strcmp(devtab[devno]->name, name) == 0)
    {
      devtab[devno]->refcnt++;
      return devno;
    }
  }

  return NODEV;
}

int dev_close(devno_t devno)
{
  if (devno < 0 || devno >= num_devices) return -1;
  if (devtab[devno]->refcnt == 0) return -1;
  devtab[devno]->refcnt--;
  return 0;
}

int dev_ioctl(devno_t devno, int cmd, void *args)
{
  struct dev *dev;

  if (devno < 0 || devno >= num_devices) return -1;
  dev = devtab[devno];

  return dev->driver->ioctl(dev, cmd, args);
}

int dev_read(devno_t devno, void *buffer, size_t count, blkno_t blkno)
{
  struct dev *dev;

  if (devno < 0 || devno >= num_devices) return -1;
  dev = devtab[devno];

  return dev->driver->read(dev, buffer, count, blkno);
}

int dev_write(devno_t devno, void *buffer, size_t count, blkno_t blkno)
{
  struct dev *dev;

  if (devno < 0 || devno >= num_devices) return -1;
  dev = devtab[devno];

  return dev->driver->write(dev, buffer, count, blkno);
}
