//
// kmem.c
//
// Copyright (c) 2001 Michael Ringgaard. All rights reserved.
//
// Kernel memory page allocator
//

#include <os/krnl.h>

#define OSVMAP_PAGES 1
#define OSVMAP_ENTRIES (OSVMAP_PAGES * PAGESIZE / sizeof(struct rmap))

struct rmap *osvmap = (struct rmap *) OSVMAP_ADDRESS;

void *alloc_pages(int pages)
{
  char *vaddr;
  int i;
  unsigned long pfn;

  vaddr = (char *) PTOB(rmap_alloc(osvmap, pages));
  for (i = 0; i < pages; i++)
  {
    pfn = alloc_pageframe(PFT_SYS);
    map_page(vaddr + PTOB(i), pfn, PT_WRITABLE);
  }

  return vaddr;
}

void *alloc_pages_align(int pages, int align)
{
  char *vaddr;
  int i;
  unsigned long pfn;

  vaddr = (char *) PTOB(rmap_alloc_align(osvmap, pages, align));
  for (i = 0; i < pages; i++)
  {
    pfn = alloc_pageframe(PFT_SYS);
    map_page(vaddr + PTOB(i), pfn, PT_WRITABLE);
  }

  return vaddr;
}

void free_pages(void *addr, int pages)
{
  int i;
  unsigned long pfn;

  for (i = 0; i < pages; i++)
  {
    pfn = BTOP(virt2phys((char *) addr + PTOB(i)));
    free_pageframe(pfn);
    unmap_page((char *) addr + PTOB(i));
  }

  rmap_free(osvmap, BTOP(addr), pages);
}

void init_kmem()
{
  int pfn;

  // Allocate page frame for kernel heap resource map and map into syspages
  pfn = alloc_pageframe(PFT_SYS);
  map_page(osvmap, pfn, PT_WRITABLE);

  // Initialize resource map for kernel heap
  rmap_init(osvmap, OSVMAP_ENTRIES);

  // Add kernel heap address space to osvmap
  rmap_free(osvmap, BTOP(KHEAPBASE), BTOP(KHEAPSIZE));
}

