//
// pic.c
//
// Copyright (c) 2001 Michael Ringgaard. All rights reserved.
//
// i8259 Programmable Interrupt Controller functions
//

#include <os/krnl.h>

//
// I/O port addresses
//

#define I8259_MSTR_CTRL		0x20
#define I8259_MSTR_MASK		0x21

#define I8259_SLV_CTRL		0xA0
#define I8259_SLV_MASK		0xA1

//
// Master commands
//

#define I8259_MSTR_ICW1		0x11
#define I8259_MSTR_ICW2		0x20
#define I8259_MSTR_ICW3		0x04
#define I8259_MSTR_ICW4		0x01
#define I8259_MSTR_DISABLE	0xFB   // All but cascade

//
// Slave commands
//

#define I8259_SLV_ICW1		0x11
#define I8259_SLV_ICW2		0x28
#define I8259_SLV_ICW3		0x02
#define I8259_SLV_ICW4		0x01
#define I8259_SLV_DISABLE	0xFF

//
// End of interrupt commands
//
#define I8259_EOI_BASE		0x60

#define I8259_EOI_TMR		0x60
#define I8259_EOI_KBD		0x61
#define I8259_EOI_CAS		0x62
#define I8259_EOI_SERALT	0x63
#define I8259_EOI_SERPRI	0x64
#define I8259_EOI_FD		0x66
#define I8259_EOI_HD		0x66

// All IRQs disabled initially except cascade

unsigned int irq_mask = 0xFFFB; 

//
// Initialize the 8259 Programmable Interrupt Controller
//

void init_pic()
{
  _outp(I8259_MSTR_CTRL, I8259_MSTR_ICW1);
  _outp(I8259_SLV_CTRL, I8259_SLV_ICW1);
  _outp(I8259_MSTR_MASK, I8259_MSTR_ICW2);
  _outp(I8259_SLV_MASK, I8259_SLV_ICW2);
  _outp(I8259_MSTR_MASK, I8259_MSTR_ICW3);
  _outp(I8259_SLV_MASK, I8259_SLV_ICW3);
  _outp(I8259_MSTR_MASK, I8259_MSTR_ICW4);
  _outp(I8259_SLV_MASK, I8259_SLV_ICW4);
  _outp(I8259_MSTR_MASK, I8259_MSTR_DISABLE);
  _outp(I8259_SLV_MASK, I8259_SLV_DISABLE);
}

//
// Enable IRQ
//

void enable_irq(unsigned int irq)
{
  irq_mask &= ~(1 << irq);
  if (irq >= 8) irq_mask &= ~(1 << 2);
	
  _outp(I8259_MSTR_MASK, irq_mask & 0xFF);
  _outp(I8259_SLV_MASK, (irq_mask >> 8) & 0xFF);
}

//
// Disable IRQ
//

void disable_irq(unsigned int irq)
{
  irq_mask |= (1 << irq);
  if ((irq_mask & 0xFF00) == 0xFF00) irq_mask |= (1 << 2);
	
  _outp(I8259_MSTR_MASK, irq_mask & 0xFF);
  _outp(I8259_SLV_MASK, (irq_mask >> 8) & 0xFF);
}

//
// Signal end of interrupt to PIC

void eoi(unsigned int irq)
{
  if (irq < 8)
    _outp(I8259_MSTR_CTRL, irq + I8259_EOI_BASE);
  else
  {
    _outp(I8259_SLV_CTRL, (irq - 8) + I8259_EOI_BASE);
    _outp(I8259_MSTR_CTRL, I8259_EOI_CAS);
  }
}
