//
// sysapi.h
//
// Copyright (c) 2001 Michael Ringgaard. All rights reserved.
//
// Operating System API
//

#include <os.h>
#include <os/syscall.h>

__declspec(naked) int syscall(int syscallno, void *params)
{
  __asm
  {
    push  ebp
    mov	  ebp, esp

    mov	  eax, 8[ebp]
    mov	  edx, 12[ebp]

    int	  48

    leave
    ret
  }
}

int format(char *devname, char *type, char *opts)
{
  return syscall(SYSCALL_FORMAT, &devname);
}

int mount(char *type, char *path, char *devname, char *opts)
{
  return syscall(SYSCALL_MOUNT, &type);
}

int unmount()
{
  return syscall(SYSCALL_UNMOUNT, NULL);
}

handle_t open(char *name, int mode)
{
  return syscall(SYSCALL_OPEN, &name);
}

int close(handle_t h)
{
  return syscall(SYSCALL_CLOSE, &h);
}

int flush(handle_t f)
{
  return syscall(SYSCALL_FLUSH, &f);
}

int read(handle_t f, void *data, size_t size)
{
  return syscall(SYSCALL_READ, &f);
}

int write(handle_t f, void *data, size_t size)
{
  return syscall(SYSCALL_WRITE, &f);
}

loff_t tell(handle_t f)
{
  return syscall(SYSCALL_TELL, &f);
}

loff_t lseek(handle_t f, loff_t offset, int origin)
{
  return syscall(SYSCALL_LSEEK, &f);
}

int chsize(handle_t f, loff_t size)
{
  return syscall(SYSCALL_CHSIZE, &f);
}

int fstat(handle_t f, struct stat *buffer)
{
  return syscall(SYSCALL_FSTAT, &f);
}

int stat(char *name, struct stat *buffer)
{
  return syscall(SYSCALL_STAT, &name);
}

int mkdir(char *name)
{
  return syscall(SYSCALL_MKDIR, &name);
}

int rmdir(char *name)
{
  return syscall(SYSCALL_RMDIR, &name);
}

int rename(char *oldname, char *newname)
{
  return syscall(SYSCALL_RENAME, &oldname);
}

int link(char *oldname, char *newname)
{
  return syscall(SYSCALL_LINK, &oldname);
}

int unlink(char *name)
{
  return syscall(SYSCALL_UNLINK, &name);
}

handle_t opendir(char *name)
{
  return syscall(SYSCALL_OPENDIR, &name);
}

int readdir(handle_t f, struct dirent *dirp, int count)
{
  return syscall(SYSCALL_READDIR, &f);
}

void *mmap(void *addr, unsigned long size, int type, int protect)
{
  return (void *) syscall(SYSCALL_MMAP, &addr);
}

int munmap(void *addr, unsigned long size, int type)
{
  return syscall(SYSCALL_MUNMAP, &addr);
}

void *mremap(void *addr, unsigned long oldsize, unsigned long newsize, int type, int protect)
{
  return (void *) syscall(SYSCALL_MREMAP, &addr);
}

int mprotect(void *addr, unsigned long size, int protect)
{
  return syscall(SYSCALL_MPROTECT, &addr);
}

int mlock(void *addr, unsigned long size)
{
  return syscall(SYSCALL_MLOCK, &addr);
}

int munlock(void *addr, unsigned long size)
{
  return syscall(SYSCALL_MUNLOCK, &addr);
}

int wait(handle_t h, int timeout)
{
  return syscall(SYSCALL_WAIT, &h);
}

handle_t mkevent(int manual_reset, int initial_state)
{
  return syscall(SYSCALL_MKEVENT, &manual_reset);
}

int epulse(handle_t h)
{
  return syscall(SYSCALL_EPULSE, &h);
}

int eset(handle_t h)
{
  return syscall(SYSCALL_ESET, &h);
}

int ereset(handle_t h)
{
  return syscall(SYSCALL_ERESET, &h);
}

handle_t self()
{
  return syscall(SYSCALL_SELF, NULL);
}

void exit()
{
  syscall(SYSCALL_EXIT, NULL);
}
